@@ -27,6 +27,35 @@
 #define DUMP_VECT 0
 #endif
 
+namespace libcdr
+{
+namespace
+{
+
+/// Move the number into [0;1] range.
+void normalize(double &d)
+{
+  if (d < 0.0)
+  {
+    const double n = d + static_cast<unsigned long>(-d) + 1.0;
+    if ((n < 0.0) || (n > 1.0))
+      d = 0.0; // The number was too big, thus rounded. Just pick a value.
+    else
+      d = n;
+  }
+  if (d > 1.0)
+  {
+    const double n = d - static_cast<unsigned long>(d);
+    if ((n < 0.0) || (n > 1.0))
+      d = 0.0; // The number was too big, thus rounded. Just pick a value.
+    else
+      d = n;
+  }
+}
+
+}
+}
+
 libcdr::CDRContentCollector::CDRContentCollector(libcdr::CDRParserState &ps, librevenge::RVNGDrawingInterface *painter) :
   m_painter(painter), m_isDocumentStarted(false),
   m_isPageProperties(false), m_isPageStarted(false), m_ignorePage(false),
@@ -429,9 +458,12 @@ void libcdr::CDRContentCollector::_flushCurrentPath()
     outputElement.addStartTextObject(textFrameProps);
     for (unsigned i = 0; i < m_currentText->size(); ++i)
     {
+      const std::vector<CDRText> &currentLine = (*m_currentText)[i].m_line;
+      if (currentLine.empty())
+        continue;
       librevenge::RVNGPropertyList paraProps;
       bool rtl = false;
-      switch ((*m_currentText)[i].m_line[0].m_charStyle.m_align)
+      switch (currentLine[0].m_charStyle.m_align)
       {
       case 1:  // Left
         if (!rtl)
@@ -458,22 +490,22 @@ void libcdr::CDRContentCollector::_flushCurrentPath()
       default:
         break;
       }
-//      paraProps.insert("fo:text-indent", (*m_currentText)[i].m_charStyle.m_firstIndent);
-//      paraProps.insert("fo:margin-left", (*m_currentText)[i].m_charStyle.m_leftIndent);
-//      paraProps.insert("fo:margin-right", (*m_currentText)[i].m_charStyle.m_rightIndent);
       outputElement.addOpenParagraph(paraProps);
-      for (unsigned j = 0; j < (*m_currentText)[i].m_line.size(); ++j)
+      for (unsigned j = 0; j < currentLine.size(); ++j)
       {
-        librevenge::RVNGPropertyList spanProps;
-        double fontSize = (double)cdr_round(144.0*(*m_currentText)[i].m_line[j].m_charStyle.m_fontSize) / 2.0;
-        spanProps.insert("fo:font-size", fontSize, librevenge::RVNG_POINT);
-        if ((*m_currentText)[i].m_line[j].m_charStyle.m_fontName.len())
-          spanProps.insert("style:font-name", (*m_currentText)[i].m_line[j].m_charStyle.m_fontName);
-        if ((*m_currentText)[i].m_line[j].m_charStyle.m_fillStyle.fillType != (unsigned short)-1)
-          spanProps.insert("fo:color", m_ps.getRGBColorString((*m_currentText)[i].m_line[j].m_charStyle.m_fillStyle.color1));
-        outputElement.addOpenSpan(spanProps);
-        outputElement.addInsertText((*m_currentText)[i].m_line[j].m_text);
-        outputElement.addCloseSpan();
+        if (!currentLine[j].m_text.empty())
+        {
+          librevenge::RVNGPropertyList spanProps;
+          double fontSize = (double)cdr_round(144.0*currentLine[j].m_charStyle.m_fontSize) / 2.0;
+          spanProps.insert("fo:font-size", fontSize, librevenge::RVNG_POINT);
+          if (currentLine[j].m_charStyle.m_fontName.len())
+            spanProps.insert("style:font-name", currentLine[j].m_charStyle.m_fontName);
+          if (currentLine[j].m_charStyle.m_fillStyle.fillType != (unsigned short)-1)
+            spanProps.insert("fo:color", m_ps.getRGBColorString(currentLine[j].m_charStyle.m_fillStyle.color1));
+          outputElement.addOpenSpan(spanProps);
+          outputElement.addInsertText(currentLine[j].m_text);
+          outputElement.addCloseSpan();
+        }
       }
       outputElement.addCloseParagraph();
     }
@@ -740,7 +772,7 @@ void libcdr::CDRContentCollector::_fillProperties(librevenge::RVNGPropertyList &
             fclose(f);
           }
 #endif
-          propList.insert("draw:fill-image", image.getBase64Data());
+          propList.insert("draw:fill-image", image);
           propList.insert("librevenge:mime-type", "image/bmp");
           propList.insert("style:repeat", "repeat");
           if (m_currentFillStyle.imageFill.isRelative)
@@ -773,19 +805,13 @@ void libcdr::CDRContentCollector::_fillProperties(librevenge::RVNGPropertyList &
             if (m_fillTransforms.getTranslateX() != 0.0)
             {
               double xOffset = m_fillTransforms.getTranslateX() / m_currentFillStyle.imageFill.width;
-              while (xOffset < 0.0)
-                xOffset += 1.0;
-              while (xOffset > 1.0)
-                xOffset -= 1.0;
+              normalize(xOffset);
               propList.insert("draw:fill-image-ref-point-x", xOffset, librevenge::RVNG_PERCENT);
             }
             if (m_fillTransforms.getTranslateY() != 0.0)
             {
               double yOffset = m_fillTransforms.getTranslateY() / m_currentFillStyle.imageFill.width;
-              while (yOffset < 0.0)
-                yOffset += 1.0;
-              while (yOffset > 1.0)
-                yOffset -= 1.0;
+              normalize(yOffset);
               propList.insert("draw:fill-image-ref-point-y", 1.0 - yOffset, librevenge::RVNG_PERCENT);
             }
           }
@@ -807,7 +833,7 @@ void libcdr::CDRContentCollector::_fillProperties(librevenge::RVNGPropertyList &
         {
           propList.insert("librevenge:mime-type", "image/bmp");
           propList.insert("draw:fill", "bitmap");
-          propList.insert("draw:fill-image", iterBmp->second.getBase64Data());
+          propList.insert("draw:fill-image", iterBmp->second);
           propList.insert("style:repeat", "repeat");
           if (m_currentFillStyle.imageFill.isRelative)
           {
@@ -867,7 +893,7 @@ void libcdr::CDRContentCollector::_fillProperties(librevenge::RVNGPropertyList &
         {
           propList.insert("draw:fill", "bitmap");
           propList.insert("librevenge:mime-type", "image/svg+xml");
-          propList.insert("draw:fill-image", iterVect->second.getBase64Data());
+          propList.insert("draw:fill-image", iterVect->second);
           propList.insert("style:repeat", "repeat");
           if (m_currentFillStyle.imageFill.isRelative)
           {
@@ -1129,7 +1155,10 @@ void libcdr::CDRContentCollector::_generateBitmapFromPattern(librevenge::RVNGBin
     while (i <lineWidth && k < width)
     {
       unsigned l = 0;
-      unsigned char c = pattern.pattern[(j-1)*lineWidth+i];
+      unsigned char c = 0;
+      const unsigned index = (j-1)*lineWidth+i;
+      if (index < pattern.pattern.size())
+        c = pattern.pattern[index];
       i++;
       while (k < width && l < 8)
       {
@@ -1185,7 +1214,10 @@ void libcdr::CDRContentCollector::collectSpnd(unsigned spnd)
 
 void libcdr::CDRContentCollector::collectVectorPattern(unsigned id, const librevenge::RVNGBinaryData &data)
 {
-  librevenge::RVNGInputStream *input = const_cast<librevenge::RVNGInputStream *>(data.getDataStream());
+  librevenge::RVNGInputStream *input = data.getDataStream();
+  if (!input)
+    return;
+
   input->seek(0, librevenge::RVNG_SEEK_SET);
   if (!libcdr::CMXDocument::isSupported(input))
     return;
