@@ -1,29 +1,10 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* libcdr
- * Version: MPL 1.1 / GPLv2+ / LGPLv2+
+/*
+ * This file is part of the libcdr project.
  *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License or as specified alternatively below. You may obtain a copy of
- * the License at http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * Major Contributor(s):
- * Copyright (C) 2012 Fridrich Strba <fridrich.strba@bluewin.ch>
- *
- * All Rights Reserved.
- *
- * For minor contributions see the git repository.
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPLv2+"), or
- * the GNU Lesser General Public License Version 2 or later (the "LGPLv2+"),
- * in which case the provisions of the GPLv2+ or the LGPLv2+ are applicable
- * instead of those above.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
 #ifndef __CDRTYPES_H__
@@ -31,22 +12,23 @@
 
 #include <vector>
 #include <math.h>
-#include <libwpd/libwpd.h>
+#include <librevenge/librevenge.h>
 #include "CDRTransforms.h"
+#include "CDRPath.h"
+#include "libcdr_utils.h"
 
 namespace libcdr
 {
-class CDRPath;
 
-struct CDRBBox
+struct CDRBox
 {
   double m_x;
   double m_y;
   double m_w;
   double m_h;
-  CDRBBox()
+  CDRBox()
     : m_x(0.0), m_y(0.0), m_w(0.0), m_h(0.0) {}
-  CDRBBox(double x0, double y0, double x1, double y1)
+  CDRBox(double x0, double y0, double x1, double y1)
     : m_x(x0 < x1 ? x0 : x1), m_y(y0 < y1 ? y0 : y1), m_w(fabs(x1-x0)), m_h(fabs(y1-y0)) {}
   double getWidth() const
   {
@@ -122,7 +104,7 @@ struct CDRFillStyle
   CDRGradient gradient;
   CDRImageFill imageFill;
   CDRFillStyle()
-    : fillType(0), color1(), color2(), gradient(), imageFill() {}
+    : fillType((unsigned short)-1), color1(), color2(), gradient(), imageFill() {}
   CDRFillStyle(unsigned short ft, CDRColor c1, CDRColor c2, const CDRGradient &gr, const CDRImageFill &img)
     : fillType(ft), color1(c1), color2(c2), gradient(gr), imageFill(img) {}
 };
@@ -137,37 +119,58 @@ struct CDRLineStyle
   double angle;
   CDRColor color;
   std::vector<unsigned> dashArray;
-  unsigned startMarkerId;
-  unsigned endMarkerId;
+  CDRPath startMarker;
+  CDRPath endMarker;
   CDRLineStyle()
-    : lineType(0), capsType(0), joinType(0), lineWidth(0.0),
+    : lineType((unsigned short)-1), capsType(0), joinType(0), lineWidth(0.0),
       stretch(0.0), angle(0.0), color(), dashArray(),
-      startMarkerId(0), endMarkerId(0) {}
+      startMarker(), endMarker() {}
   CDRLineStyle(unsigned short lt, unsigned short ct, unsigned short jt,
                double lw, double st, double a, const CDRColor &c, const std::vector<unsigned> &da,
-               unsigned smi, unsigned emi)
+               const CDRPath &sm, const CDRPath &em)
     : lineType(lt), capsType(ct), joinType(jt), lineWidth(lw),
       stretch(st), angle(a), color(c), dashArray(da),
-      startMarkerId(smi), endMarkerId(emi) {}
+      startMarker(sm), endMarker(em) {}
 };
 
 struct CDRCharacterStyle
 {
   unsigned short m_charSet;
-  unsigned short m_fontId;
+  librevenge::RVNGString m_fontName;
   double m_fontSize;
+  unsigned m_align;
+  double m_leftIndent, m_firstIndent, m_rightIndent;
+  CDRLineStyle m_lineStyle;
+  CDRFillStyle m_fillStyle;
+  unsigned m_parentId;
   CDRCharacterStyle()
-    : m_charSet(0), m_fontId(0), m_fontSize(0.0) {}
-  CDRCharacterStyle(unsigned short charSet, unsigned short fontId, double fontSize)
-    : m_charSet(charSet), m_fontId(fontId), m_fontSize(fontSize) {}
+    : m_charSet((unsigned short)-1), m_fontName(),
+      m_fontSize(0.0), m_align(0), m_leftIndent(0.0), m_firstIndent(0.0),
+      m_rightIndent(0.0), m_lineStyle(), m_fillStyle(), m_parentId(0)
+  {
+    m_fontName.clear();
+  }
   void overrideCharacterStyle(const CDRCharacterStyle &override)
   {
-    if (override.m_charSet)
+    if (override.m_charSet != (unsigned short)-1 || override.m_fontName.len())
+    {
       m_charSet = override.m_charSet;
-    if (override.m_fontId)
-      m_fontId = override.m_fontId;
-    if (override.m_fontSize > 0.0)
+      m_fontName = override.m_fontName;
+    }
+    if (!CDR_ALMOST_ZERO(override.m_fontSize))
       m_fontSize = override.m_fontSize;
+    if (override.m_align)
+      m_align = override.m_align;
+    if (override.m_leftIndent != 0.0 && override.m_firstIndent != 0.0 && override.m_rightIndent != 0.0)
+    {
+      m_leftIndent = override.m_leftIndent;
+      m_firstIndent = override.m_firstIndent;
+      m_rightIndent = override.m_rightIndent;
+    }
+    if (override.m_lineStyle.lineType != (unsigned short)-1)
+      m_lineStyle = override.m_lineStyle;
+    if (override.m_fillStyle.fillType != (unsigned short)-1)
+      m_fillStyle = override.m_fillStyle;
   }
 };
 
@@ -187,13 +190,13 @@ struct CDRPolygon
 
 struct CDRImage
 {
-  WPXBinaryData m_image;
+  librevenge::RVNGBinaryData m_image;
   double m_x1;
   double m_x2;
   double m_y1;
   double m_y2;
   CDRImage() : m_image(), m_x1(0.0), m_x2(0.0), m_y1(0.0), m_y2(0.0) {}
-  CDRImage(const WPXBinaryData &image, double x1, double x2, double y1, double y2)
+  CDRImage(const librevenge::RVNGBinaryData &image, double x1, double x2, double y1, double y2)
     : m_image(image), m_x1(x1), m_x2(x2), m_y1(y1), m_y2(y2) {}
   double getMiddleX() const
   {
@@ -203,15 +206,7 @@ struct CDRImage
   {
     return (m_y1 + m_y2) / 2.0;
   }
-  double getWidth() const
-  {
-    return fabs(m_x1 - m_x2);
-  }
-  double getHeight() const
-  {
-    return fabs(m_y1 - m_y2);
-  }
-  const WPXBinaryData &getImage() const
+  const librevenge::RVNGBinaryData &getImage() const
   {
     return m_image;
   }
@@ -292,7 +287,6 @@ struct WaldoRecordType1
 
 struct CDRCMYKColor
 {
-  CDRCMYKColor(unsigned colorValue, bool percentage = true);
   CDRCMYKColor(double cyan, double magenta, double yellow, double black)
     : c(cyan), m(magenta), y(yellow), k(black) {}
   ~CDRCMYKColor() {}
@@ -300,58 +294,73 @@ struct CDRCMYKColor
   double m;
   double y;
   double k;
-  void applyTint(double tint);
-  unsigned getColorValue() const;
 };
 
 struct CDRRGBColor
 {
-  CDRRGBColor(unsigned colorValue);
   CDRRGBColor(double red, double green, double blue)
     : r(red), g(green), b(blue) {}
   ~CDRRGBColor() {}
   double r;
   double g;
   double b;
-  void applyTint(double tint);
-  unsigned getColorValue() const;
 };
 
 struct CDRLab2Color
 {
-  CDRLab2Color(unsigned colorValue);
   CDRLab2Color(double l, double A, double B)
     : L(l), a(A), b(B) {}
   ~CDRLab2Color() {}
   double L;
   double a;
   double b;
-  void applyTint(double tint);
-  unsigned getColorValue() const;
 };
 
 struct CDRLab4Color
 {
-  CDRLab4Color(unsigned colorValue);
   CDRLab4Color(double l, double A, double B)
     : L(l), a(A), b(B) {}
   ~CDRLab4Color() {}
   double L;
   double a;
   double b;
-  void applyTint(double tint);
-  unsigned getColorValue() const;
 };
 
 struct CDRText
 {
   CDRText() : m_text(), m_charStyle() {}
-  CDRText(const WPXString &text, const CDRCharacterStyle &charStyle)
+  CDRText(const librevenge::RVNGString &text, const CDRCharacterStyle &charStyle)
     : m_text(text), m_charStyle(charStyle) {}
-  WPXString m_text;
+  librevenge::RVNGString m_text;
   CDRCharacterStyle m_charStyle;
 };
 
+struct CDRTextLine
+{
+  CDRTextLine() : m_line() {}
+  CDRTextLine(const CDRTextLine &line) : m_line(line.m_line) {}
+  void append(const CDRText &text)
+  {
+    m_line.push_back(text);
+  }
+  void clear()
+  {
+    m_line.clear();
+  }
+  std::vector<CDRText> m_line;
+};
+
+struct CDRFont
+{
+  CDRFont() : m_name(), m_encoding(0) {}
+  CDRFont(const librevenge::RVNGString &name, unsigned short encoding)
+    : m_name(name), m_encoding(encoding) {}
+  CDRFont(const CDRFont &font)
+    : m_name(font.m_name), m_encoding(font.m_encoding) {}
+  librevenge::RVNGString m_name;
+  unsigned short m_encoding;
+};
+
 } // namespace libcdr
 
 #endif /* __CDRTYPES_H__ */
